package app.alan.alan_voice

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import android.view.View
import android.view.Gravity
import android.widget.FrameLayout
import android.app.Activity
import android.content.res.Resources
import android.view.ViewGroup
import com.alan.alansdk.button.AlanButton

class AlanVoicePlugin(private val activity: Activity): MethodCallHandler {

  private var alanButton: AlanButton? = null

  companion object {

    private val ARGUMENT_PROJECT_ID = "projectId"

    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "alan_voice")
      channel.setMethodCallHandler(AlanVoicePlugin(registrar.activity()))
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "getVersion" -> version(result)
      "createButton" -> createButton(result)
      "initButton" -> initButton(result, call)
      "showButton" -> showButton(result)
      else -> result.notImplemented()
    }
  }

  private fun version(result: Result) {
    if (alanButton != null) {
      result.success(alanButton?.sdk?.version ?: "")
    } else {
      result.error("No alan button", null, null)
    }
  }

  private fun showButton(result: Result) {
    alanButton?.showButton()
    result.success(true)
  }

  private fun initButton(result: Result, call: MethodCall) {
    if (alanButton == null) {
      createButton(null)
    }
    val projectId = call.argument<String>(ARGUMENT_PROJECT_ID)
    if (projectId != null) {
      alanButton?.initSDK(projectId)
      alanButton?.showButton()
      result.success(true)
    } else {
      result.error("No objectId, please provide objectId argument", null, null)
    }
  }


  private fun createButton(result: Result?) {
    val rootView = activity.findViewById(android.R.id.content) as ViewGroup

    alanButton = AlanButton(activity, null)
    val params = FrameLayout.LayoutParams(dpToPx(100), dpToPx(100))
    params.gravity = Gravity.BOTTOM or Gravity.END
    alanButton?.let {
      it.layoutParams = params
      it.visibility = View.GONE
      it.isClickable = true
      rootView.addView(it)
    }

    result?.success(true)
  }

  private fun dpToPx(dp: Int): Int {
    return (dp * Resources.getSystem().displayMetrics.density).toInt()
  }
}
